@@ -23,13 +23,18 @@ from octavia.amphorae.drivers.health import heartbeat_udp
 from octavia.common import service
 from octavia.controller.healthmanager import health_manager
 from octavia.controller.healthmanager import update_db
-from octavia.i18n import _LI
 from octavia import version
 
 
 CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
-CONF.import_group('health_manager', 'octavia.common.config')
+
+
+# Used for while true loops to allow testing
+# TODO(johnsom) This will be removed with
+#               https://review.openstack.org/#/c/456420/
+def true_func():
+    return True
 
 
 def hm_listener():
@@ -43,8 +48,12 @@ def hm_listener():
 
 def hm_health_check():
     hm = health_manager.HealthManager()
-    while True:
-        hm.health_check()
+    while true_func():
+        try:
+            hm.health_check()
+        except Exception as e:
+            LOG.warning('Health Manager caught the following exception and '
+                        'is restarting: {}'.format(e))
 
 
 def main():
@@ -60,15 +69,15 @@ def main():
     hm_health_check_proc = multiprocessing.Process(name='HM_health_check',
                                                    target=hm_health_check)
     processes.append(hm_health_check_proc)
-    LOG.info(_LI("Health Manager listener process starts:"))
+    LOG.info("Health Manager listener process starts:")
     hm_listener_proc.start()
-    LOG.info(_LI("Health manager check process starts:"))
+    LOG.info("Health manager check process starts:")
     hm_health_check_proc.start()
 
     try:
         for process in processes:
             process.join()
     except KeyboardInterrupt:
-        LOG.info(_LI("Health Manager existing due to signal"))
+        LOG.info("Health Manager existing due to signal")
         hm_listener_proc.terminate()
         hm_health_check_proc.terminate()
