@@ -25,40 +25,78 @@ import oslo_messaging as messaging
 
 from octavia.common import constants
 from octavia.common import utils
-from octavia.i18n import _LI
 from octavia import version
 
 LOG = logging.getLogger(__name__)
 
+# TODO(rm_work) Remove in or after "R" release
+API_SETTINGS_DEPRECATION_MESSAGE = _(
+    'This setting has moved to the [api_settings] section.')
+
 core_opts = [
+    cfg.HostnameOpt('host', default=utils.get_hostname(),
+                    help=_("The hostname Octavia is running on")),
+    cfg.StrOpt('octavia_plugins', default='hot_plug_plugin',
+               help=_("Name of the controller plugin to use")),
+
+    # TODO(johnsom) Remove in or after "R" release
+    cfg.IPOpt('bind_host', help=_("The host IP to bind to"),
+              deprecated_for_removal=True,
+              deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
+    # TODO(johnsom) Remove in or after "R" release
+    cfg.PortOpt('bind_port', help=_("The port to bind to"),
+                deprecated_for_removal=True,
+                deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
+    # TODO(johnsom) Remove in or after "R" release
+    cfg.StrOpt('auth_strategy',
+               choices=[constants.NOAUTH,
+                        constants.KEYSTONE,
+                        constants.TESTING],
+               help=_("The auth strategy for API requests."),
+               deprecated_for_removal=True,
+               deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
+    # TODO(johnsom) Remove in or after "R" release
+    cfg.StrOpt('api_handler',
+               help=_("The handler that the API communicates with"),
+               deprecated_for_removal=True,
+               deprecated_reason=API_SETTINGS_DEPRECATION_MESSAGE),
+]
+
+api_opts = [
     cfg.IPOpt('bind_host', default='127.0.0.1',
               help=_("The host IP to bind to")),
     cfg.PortOpt('bind_port', default=9876,
                 help=_("The port to bind to")),
-    cfg.StrOpt('auth_strategy', default=constants.NOAUTH,
-               choices=[constants.NOAUTH, constants.KEYSTONE],
+    cfg.StrOpt('auth_strategy', default=constants.KEYSTONE,
+               choices=[constants.NOAUTH,
+                        constants.KEYSTONE,
+                        constants.TESTING],
                help=_("The auth strategy for API requests.")),
     cfg.StrOpt('api_handler', default='queue_producer',
                help=_("The handler that the API communicates with")),
-    cfg.StrOpt('api_paste_config', default="api-paste.ini",
-               help=_("The API paste config file to use")),
-    cfg.StrOpt('api_extensions_path', default="",
-               help=_("The path for API extensions")),
-    cfg.BoolOpt('allow_bulk', default=True,
-                help=_("Allow the usage of the bulk API")),
-    cfg.BoolOpt('allow_pagination', default=False,
-                help=_("Allow the usage of the pagination")),
-    cfg.BoolOpt('allow_sorting', default=False,
-                help=_("Allow the usage of the sorting")),
-    cfg.StrOpt('pagination_max_limit', default="-1",
+    cfg.BoolOpt('allow_pagination', default=True,
+                help=_("Allow the usage of pagination")),
+    cfg.BoolOpt('allow_sorting', default=True,
+                help=_("Allow the usage of sorting")),
+    cfg.BoolOpt('allow_filtering', default=True,
+                help=_("Allow the usage of filtering")),
+    cfg.BoolOpt('allow_field_selection', default=True,
+                help=_("Allow the usage of field selection")),
+    cfg.StrOpt('pagination_max_limit',
+               default=str(constants.DEFAULT_PAGE_SIZE),
                help=_("The maximum number of items returned in a single "
                       "response. The string 'infinite' or a negative "
                       "integer value means 'no limit'")),
-    cfg.StrOpt('host', default=utils.get_hostname(),
-               help=_("The hostname Octavia is running on")),
-    cfg.StrOpt('octavia_plugins',
-               default='hot_plug_plugin',
-               help=_('Name of the controller plugin to use'))
+    cfg.StrOpt('api_base_uri',
+               help=_("Base URI for the API for use in pagination links. "
+                      "This will be autodetected from the request if not "
+                      "overridden here.")),
+    cfg.BoolOpt('api_v1_enabled', default=True,
+                help=_("Expose the v1 API?")),
+    cfg.BoolOpt('api_v2_enabled', default=True,
+                help=_("Expose the v2 API?")),
+    cfg.BoolOpt('allow_tls_terminated_listeners', default=True,
+                help=_("Allow users to create TLS Terminated listeners?")),
 ]
 
 # Options only used by the amphora agent
@@ -83,7 +121,6 @@ amphora_agent_opts = [
 ]
 
 networking_opts = [
-    cfg.StrOpt('lb_network_name', help=_('Name of amphora internal network')),
     cfg.IntOpt('max_retries', default=15,
                help=_('The maximum attempts to retry an action with the '
                       'networking service.')),
@@ -92,7 +129,17 @@ networking_opts = [
                       'networking service.')),
     cfg.IntOpt('port_detach_timeout', default=300,
                help=_('Seconds to wait for a port to detach from an '
-                      'amphora.'))
+                      'amphora.')),
+    cfg.BoolOpt('allow_vip_network_id', default=True,
+                help=_('Can users supply a network_id for their VIP?')),
+    cfg.BoolOpt('allow_vip_subnet_id', default=True,
+                help=_('Can users supply a subnet_id for their VIP?')),
+    cfg.BoolOpt('allow_vip_port_id', default=True,
+                help=_('Can users supply a port_id for their VIP?')),
+    cfg.ListOpt('valid_vip_networks',
+                help=_('List of network_ids that are valid for VIP '
+                       'creation. If this field is empty, no validation '
+                       'is performed.')),
 ]
 
 healthmanager_opts = [
@@ -136,7 +183,9 @@ healthmanager_opts = [
                       'don\'t need to sync the database or are running '
                       'octavia in stand alone mode use the '
                       'noop_event_streamer'),
-               default='noop_event_streamer')]
+               default='noop_event_streamer'),
+    cfg.BoolOpt('sync_provisioning_status', default=False,
+                help=_("Enable provisioning status sync with neutron db"))]
 
 oslo_messaging_opts = [
     cfg.StrOpt('topic'),
@@ -160,8 +209,21 @@ haproxy_amphora_opts = [
                default=5,
                help=_('Retry timeout between connection attempts in '
                       'seconds.')),
-    cfg.StrOpt('user_group',
-               default='nogroup',
+    cfg.IntOpt('build_rate_limit',
+               default=-1,
+               help=_('Number of amphorae that could be built per controller'
+                      'worker, simultaneously.')),
+    cfg.IntOpt('build_active_retries',
+               default=300,
+               help=_('Retry threshold for waiting for a build slot for '
+                      'an amphorae.')),
+    cfg.IntOpt('build_retry_interval',
+               default=5,
+               help=_('Retry timeout between build attempts in '
+                      'seconds.')),
+    cfg.StrOpt('user_group', deprecated_for_removal=True,
+               deprecated_reason='This is now automatically discovered '
+                                 ' and configured.',
                help=_('The user group for haproxy to run under inside the '
                       'amphora.')),
     cfg.StrOpt('haproxy_stick_size', default='10k',
@@ -202,6 +264,9 @@ haproxy_amphora_opts = [
 ]
 
 controller_worker_opts = [
+    cfg.IntOpt('workers',
+               default=1, min=1,
+               help='Number of workers for the controller-worker service.'),
     cfg.IntOpt('amp_active_retries',
                default=10,
                help=_('Retry attempts to wait for Amphora to become active')),
@@ -239,11 +304,6 @@ controller_worker_opts = [
                 help=_('List of networks to attach to the Amphorae. '
                        'All networks defined in the list will '
                        'be attached to each amphora.')),
-    cfg.StrOpt('amp_network',
-               deprecated_for_removal=True,
-               deprecated_reason='Replaced by amp_boot_network_list.',
-               default='',
-               help=_('Network to attach to the Amphorae.')),
     cfg.ListOpt('amp_secgroup_list',
                 default='',
                 help=_('List of security groups to attach to the Amphora.')),
@@ -278,7 +338,12 @@ task_flow_opts = [
                help=_('TaskFlow engine to use')),
     cfg.IntOpt('max_workers',
                default=5,
-               help=_('The maximum number of workers'))
+               help=_('The maximum number of workers')),
+    cfg.BoolOpt('disable_revert', default=False,
+                help=_('If True, disables the controller worker taskflow '
+                       'flows from reverting.  This will leave resources in '
+                       'an inconsistent state and should only be used for '
+                       'debugging purposes.'))
 ]
 
 core_cli_opts = []
@@ -392,12 +457,21 @@ nova_opts = [
                help=_('CA certificates file path')),
     cfg.BoolOpt('insecure',
                 default=False,
-                help=_('Disable certificate validation on SSL connections ')),
+                help=_('Disable certificate validation on SSL connections')),
     cfg.BoolOpt('enable_anti_affinity', default=False,
                 help=_('Flag to indicate if nova anti-affinity feature is '
-                       'turned on.'))
+                       'turned on.')),
+    cfg.StrOpt('anti_affinity_policy', default=constants.ANTI_AFFINITY,
+               choices=[constants.ANTI_AFFINITY, constants.SOFT_ANTI_AFFINITY],
+               help=_('Sets the anti-affinity policy for nova')),
+    cfg.IntOpt('random_amphora_name_length', default=0,
+               help=_('If non-zero, generate a random name of the length '
+                      'provided for each amphora, in the format "a[A-Z0-9]*". '
+                      'Otherwise, the default name format will be used: '
+                      '"amphora-{UUID}".')),
+    cfg.StrOpt('availability_zone', default=None,
+               help=_('Availability zone to use for creating Amphorae')),
 ]
-
 neutron_opts = [
     cfg.StrOpt('service_name',
                help=_('The name of the neutron service in the '
@@ -455,6 +529,7 @@ quota_opts = [
 
 # Register the configuration options
 cfg.CONF.register_opts(core_opts)
+cfg.CONF.register_opts(api_opts, group='api_settings')
 cfg.CONF.register_opts(amphora_agent_opts, group='amphora_agent')
 cfg.CONF.register_opts(networking_opts, group='networking')
 cfg.CONF.register_opts(oslo_messaging_opts, group='oslo_messaging')
@@ -462,7 +537,6 @@ cfg.CONF.register_opts(haproxy_amphora_opts, group='haproxy_amphora')
 cfg.CONF.register_opts(controller_worker_opts, group='controller_worker')
 cfg.CONF.register_opts(keepalived_vrrp_opts, group='keepalived_vrrp')
 cfg.CONF.register_opts(task_flow_opts, group='task_flow')
-cfg.CONF.register_opts(oslo_messaging_opts, group='oslo_messaging')
 cfg.CONF.register_opts(house_keeping_opts, group='house_keeping')
 cfg.CONF.register_opts(anchor_opts, group='anchor')
 cfg.CONF.register_cli_opts(core_cli_opts)
@@ -492,6 +566,7 @@ def init(args, **kwargs):
     cfg.CONF(args=args, project='octavia',
              version='%%prog %s' % version.version_info.release_string(),
              **kwargs)
+    handle_deprecation_compatibility()
 
 
 def setup_logging(conf):
@@ -501,4 +576,27 @@ def setup_logging(conf):
     """
     product_name = "octavia"
     logging.setup(conf, product_name)
-    LOG.info(_LI("Logging enabled!"))
+    LOG.info("Logging enabled!")
+
+
+# Use cfg.CONF.set_default to override the new configuration setting
+# default value.  This allows a value set, at the new location, to override
+# a value set in the previous location while allowing settings that have
+# not yet been moved to be utilized.
+def handle_deprecation_compatibility():
+    # TODO(johnsom) Remove in or after "R" release
+    if cfg.CONF.bind_host is not None:
+        cfg.CONF.set_default('bind_host', cfg.CONF.bind_host,
+                             group='api_settings')
+    # TODO(johnsom) Remove in or after "R" release
+    if cfg.CONF.bind_port is not None:
+        cfg.CONF.set_default('bind_port', cfg.CONF.bind_port,
+                             group='api_settings')
+    # TODO(johnsom) Remove in or after "R" release
+    if cfg.CONF.auth_strategy is not None:
+        cfg.CONF.set_default('auth_strategy', cfg.CONF.auth_strategy,
+                             group='api_settings')
+    # TODO(johnsom) Remove in or after "R" release
+    if cfg.CONF.api_handler is not None:
+        cfg.CONF.set_default('api_handler', cfg.CONF.api_handler,
+                             group='api_settings')
