@@ -23,7 +23,6 @@ import six
 from octavia.common import clients
 from octavia.common import constants
 from octavia.common import data_models
-from octavia.i18n import _LE, _LI, _LW
 from octavia.network import base
 from octavia.network import data_models as n_data_models
 from octavia.network.drivers.neutron import base as neutron_base
@@ -38,9 +37,6 @@ VIP_SECURITY_GRP_PREFIX = 'lb-'
 OCTAVIA_OWNER = 'Octavia'
 
 CONF = cfg.CONF
-CONF.import_group('nova', 'octavia.common.config')
-CONF.import_group('controller_worker', 'octavia.common.config')
-CONF.import_group('networking', 'octavia.common.config')
 
 
 class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
@@ -77,31 +73,36 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                     ret.append(interface)
         return ret
 
-    def _get_plugged_interface(self, compute_id, network_id):
+    def _get_plugged_interface(self, compute_id, network_id, lb_network_ip):
         interfaces = self.get_plugged_networks(compute_id)
         for interface in interfaces:
-            if interface.network_id == network_id:
+            is_correct_interface = interface.network_id == network_id
+            for ip in interface.fixed_ips:
+                if ip.ip_address == lb_network_ip:
+                    is_correct_interface = False
+            if is_correct_interface:
                 return interface
 
-    def _plug_amphora_vip(self, amphora, network_id):
+    def _plug_amphora_vip(self, amphora, subnet):
         # We need a vip port owned by Octavia for Act/Stby and failover
         try:
             port = {'port': {'name': 'octavia-lb-vrrp-' + amphora.id,
-                             'network_id': network_id,
+                             'network_id': subnet.network_id,
+                             'fixed_ips': [{'subnet_id': subnet.id}],
                              'admin_state_up': True,
                              'device_owner': OCTAVIA_OWNER}}
             new_port = self.neutron_client.create_port(port)
             new_port = utils.convert_port_dict_to_model(new_port)
 
-            LOG.debug('Created vip port: {port_id} for amphora: {amp}'.format(
-                port_id=new_port.id, amp=amphora.id))
+            LOG.debug('Created vip port: %(port_id)s for amphora: %(amp)s',
+                      {'port_id': new_port.id, 'amp': amphora.id})
 
             interface = self.plug_port(amphora, new_port)
         except Exception:
-            message = _LE('Error plugging amphora (compute_id: {compute_id}) '
-                          'into vip network {network_id}.').format(
-                              compute_id=amphora.compute_id,
-                              network_id=network_id)
+            message = _('Error plugging amphora (compute_id: {compute_id}) '
+                        'into vip network {network_id}.').format(
+                            compute_id=amphora.compute_id,
+                            network_id=subnet.network_id)
             LOG.exception(message)
             raise base.PlugVIPException(message)
         return interface
@@ -110,11 +111,11 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         try:
             self._add_allowed_address_pair_to_port(port_id, vip_address)
         except neutron_client_exceptions.PortNotFoundClient as e:
-                raise base.PortNotFound(e.message)
+            raise base.PortNotFound(e.message)
         except Exception:
-            message = _LE('Error adding allowed address pair {ip} '
-                          'to port {port_id}.').format(ip=vip_address,
-                                                       port_id=port_id)
+            message = _('Error adding allowed address pair {ip} '
+                        'to port {port_id}.').format(ip=vip_address,
+                                                     port_id=port_id)
             LOG.exception(message)
             raise base.PlugVIPException(message)
 
@@ -126,7 +127,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
     def _get_ethertype_for_ip(self, ip):
         address = ipaddress.ip_address(
-            ip if six.text_type == type(ip) else six.u(ip))
+            ip if isinstance(ip, six.text_type) else six.u(ip))
         return 'IPv6' if address.version is 6 else 'IPv4'
 
     def _update_security_group_rules(self, load_balancer, sec_grp_id):
@@ -151,7 +152,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                      # with the egress rules.  VRRP uses protocol
                      # 51 and 112
                      if rule.get('direction') != 'egress' and
-                     rule.get('protocol').lower() == 'tcp']
+                     rule.get('protocol', '').lower() == 'tcp']
         add_ports = set(updated_ports) - set(old_ports)
         del_ports = set(old_ports) - set(updated_ports)
         for rule in rules.get('security_group_rules', []):
@@ -220,20 +221,20 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         while attempts <= CONF.networking.max_retries:
             try:
                 self.neutron_client.delete_security_group(sec_grp)
-                LOG.info(_LI("Deleted security group %s"), sec_grp)
+                LOG.info("Deleted security group %s", sec_grp)
                 return
             except neutron_client_exceptions.NotFound:
-                LOG.info(_LI("Security group %s not found, will assume it is "
-                             "already deleted"), sec_grp)
+                LOG.info("Security group %s not found, will assume it is "
+                         "already deleted", sec_grp)
                 return
             except Exception:
-                LOG.warning(_LW("Attempt %(attempt)s to remove security group "
-                                "%(sg)s failed."),
+                LOG.warning("Attempt %(attempt)s to remove security group "
+                            "%(sg)s failed.",
                             {'attempt': attempts + 1, 'sg': sec_grp})
             attempts += 1
             time.sleep(CONF.networking.retry_interval)
-        message = _LE("All attempts to remove security group {0} have "
-                      "failed.").format(sec_grp)
+        message = _("All attempts to remove security group {0} have "
+                    "failed.").format(sec_grp)
         LOG.exception(message)
         raise base.DeallocateVIPException(message)
 
@@ -247,7 +248,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             if sec_grp:
                 sec_grp = sec_grp.get('id')
                 LOG.info(
-                    _LI("Removing security group %(sg)s from port %(port)s"),
+                    "Removing security group %(sg)s from port %(port)s",
                     {'sg': sec_grp, 'port': vip.port_id})
                 raw_port = self.neutron_client.show_port(port.id)
                 sec_grps = raw_port.get('port', {}).get('security_groups', [])
@@ -262,14 +263,14 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         This can happen if a failover has occurred.
         """
-        try:
-            for amphora in six.moves.filter(self._filter_amphora,
-                                            vip.load_balancer.amphorae):
+        for amphora in six.moves.filter(self._filter_amphora,
+                                        vip.load_balancer.amphorae):
+            try:
                 self.neutron_client.delete_port(amphora.vrrp_port_id)
-        except (neutron_client_exceptions.NotFound,
-                neutron_client_exceptions.PortNotFoundClient):
-            LOG.debug('VIP instance port {0} already deleted.  '
-                      'Skipping.'.format(amphora.vrrp_port_id))
+            except (neutron_client_exceptions.NotFound,
+                    neutron_client_exceptions.PortNotFoundClient):
+                LOG.debug('VIP instance port %s already deleted. Skipping.',
+                          amphora.vrrp_port_id)
 
         try:
             port = self.get_port(vip.port_id)
@@ -284,13 +285,13 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             try:
                 self.neutron_client.delete_port(vip.port_id)
             except Exception:
-                message = _LE('Error deleting VIP port_id {port_id} from '
-                              'neutron').format(port_id=vip.port_id)
+                message = _('Error deleting VIP port_id {port_id} from '
+                            'neutron').format(port_id=vip.port_id)
                 LOG.exception(message)
                 raise base.DeallocateVIPException(message)
         else:
-            LOG.info(_LI("Port %s will not be deleted by Octavia as it was "
-                         "not created by Octavia."), vip.port_id)
+            LOG.info("Port %s will not be deleted by Octavia as it was "
+                     "not created by Octavia.", vip.port_id)
 
     def plug_vip(self, load_balancer, vip):
         if self.sec_grp_enabled:
@@ -301,10 +302,10 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 load_balancer.amphorae):
 
-            interface = self._get_plugged_interface(amphora.compute_id,
-                                                    subnet.network_id)
+            interface = self._get_plugged_interface(
+                amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
             if not interface:
-                interface = self._plug_amphora_vip(amphora, subnet.network_id)
+                interface = self._plug_amphora_vip(amphora, subnet)
 
             self._add_vip_address_pair(interface.port_id, vip.ip_address)
             if self.sec_grp_enabled:
@@ -312,7 +313,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                                                      interface.port_id)
             vrrp_ip = None
             for fixed_ip in interface.fixed_ips:
-                if fixed_ip.subnet_id == subnet.id:
+                is_correct_subnet = fixed_ip.subnet_id == subnet.id
+                is_management_ip = fixed_ip.ip_address == amphora.lb_network_ip
+                if is_correct_subnet and not is_management_ip:
                     vrrp_ip = fixed_ip.ip_address
                     break
             plugged_amphorae.append(data_models.Amphora(
@@ -325,31 +328,32 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         return plugged_amphorae
 
     def allocate_vip(self, load_balancer):
-        if not load_balancer.vip.port_id and not load_balancer.vip.subnet_id:
-            raise base.AllocateVIPException('Cannot allocate a vip '
-                                            'without a port_id or '
-                                            'a subnet_id.')
         if load_balancer.vip.port_id:
-            LOG.info(_LI('Port %s already exists. Nothing to be done.'),
+            LOG.info('Port %s already exists. Nothing to be done.',
                      load_balancer.vip.port_id)
             port = self.get_port(load_balancer.vip.port_id)
             return self._port_to_vip(port, load_balancer)
 
-        # Must retrieve the network_id from the subnet
-        subnet = self.get_subnet(load_balancer.vip.subnet_id)
-
+        fixed_ip = {}
+        if load_balancer.vip.subnet_id:
+            fixed_ip['subnet_id'] = load_balancer.vip.subnet_id
+        if load_balancer.vip.ip_address:
+            fixed_ip['ip_address'] = load_balancer.vip.ip_address
         # It can be assumed that network_id exists
         port = {'port': {'name': 'octavia-lb-' + load_balancer.id,
-                         'network_id': subnet.network_id,
+                         'network_id': load_balancer.vip.network_id,
                          'admin_state_up': False,
                          'device_id': 'lb-{0}'.format(load_balancer.id),
                          'device_owner': OCTAVIA_OWNER}}
+
+        if fixed_ip:
+            port['port']['fixed_ips'] = [fixed_ip]
         try:
             new_port = self.neutron_client.create_port(port)
         except Exception:
-            message = _LE('Error creating neutron port on network '
-                          '{network_id}.').format(
-                network_id=subnet.network_id)
+            message = _('Error creating neutron port on network '
+                        '{network_id}.').format(
+                network_id=load_balancer.vip.network_id)
             LOG.exception(message)
             raise base.AllocateVIPException(message)
         new_port = utils.convert_port_dict_to_model(new_port)
@@ -359,20 +363,20 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         try:
             subnet = self.get_subnet(vip.subnet_id)
         except base.SubnetNotFound:
-            msg = _LE("Can't unplug vip because vip subnet {0} was not "
-                      "found").format(vip.subnet_id)
+            msg = ("Can't unplug vip because vip subnet {0} was not "
+                   "found").format(vip.subnet_id)
             LOG.exception(msg)
             raise base.PluggedVIPNotFound(msg)
         for amphora in six.moves.filter(
             lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                 load_balancer.amphorae):
 
-            interface = self._get_plugged_interface(amphora.compute_id,
-                                                    subnet.network_id)
+            interface = self._get_plugged_interface(
+                amphora.compute_id, subnet.network_id, amphora.lb_network_ip)
             if not interface:
                 # Thought about raising PluggedVIPNotFound exception but
                 # then that wouldn't evaluate all amphorae, so just continue
-                LOG.debug(_LI('Cannot get amphora %s interface, skipped'),
+                LOG.debug('Cannot get amphora %s interface, skipped',
                           amphora.compute_id)
                 continue
             try:
@@ -386,9 +390,9 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 self.neutron_client.update_port(interface.port_id,
                                                 aap_update)
             except Exception:
-                message = _LE('Error unplugging VIP. Could not clear '
-                              'allowed address pairs from port '
-                              '{port_id}.').format(port_id=vip.port_id)
+                message = _('Error unplugging VIP. Could not clear '
+                            'allowed address pairs from port '
+                            '{port_id}.').format(port_id=vip.port_id)
                 LOG.exception(message)
                 raise base.UnplugVIPException(message)
 
@@ -400,9 +404,8 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             except base.PortNotFound:
                 pass
             except Exception as e:
-                LOG.error(_LE('Failed to delete port.  Resources may still '
-                              'be in use for port: %(port)s due to '
-                              'error: %s(except)s'),
+                LOG.error('Failed to delete port.  Resources may still be in '
+                          'use for port: %(port)s due to error: %s(except)s',
                           {'port': amphora.vrrp_port_id, 'except': e})
 
     def plug_network(self, compute_id, network_id, ip_address=None):
@@ -418,10 +421,10 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             else:
                 raise base.PlugNetworkException(e.message)
         except Exception:
-            message = _LE('Error plugging amphora (compute_id: {compute_id}) '
-                          'into network {network_id}.').format(
-                              compute_id=compute_id,
-                              network_id=network_id)
+            message = _('Error plugging amphora (compute_id: {compute_id}) '
+                        'into network {network_id}.').format(
+                            compute_id=compute_id,
+                            network_id=network_id)
             LOG.exception(message)
             raise base.PlugNetworkException(message)
 
@@ -441,20 +444,20 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 self.nova_client.servers.interface_detach(
                     server=compute_id, port_id=unplugger.port_id)
         except Exception:
-            message = _LE('Error unplugging amphora {amphora_id} from network '
-                          '{network_id}.').format(amphora_id=compute_id,
-                                                  network_id=network_id)
+            message = _('Error unplugging amphora {amphora_id} from network '
+                        '{network_id}.').format(amphora_id=compute_id,
+                                                network_id=network_id)
             if len(unpluggers) > 1:
-                message = _LE('{base} Other interfaces have been successfully '
-                              'unplugged: ').format(base=message)
+                message = _('{base} Other interfaces have been successfully '
+                            'unplugged: ').format(base=message)
                 unpluggeds = unpluggers[:index]
                 for unplugged in unpluggeds:
-                    message = _LE('{base} neutron port '
-                                  '{port_id} ').format(
-                                      base=message, port_id=unplugged.port_id)
+                    message = _('{base} neutron port '
+                                '{port_id} ').format(
+                                    base=message, port_id=unplugged.port_id)
             else:
-                message = _LE('{base} No other networks were '
-                              'unplugged.').format(base=message)
+                message = _('{base} No other networks were '
+                            'unplugged.').format(base=message)
             LOG.exception(message)
             raise base.UnplugNetworkException(message)
 
@@ -463,6 +466,10 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
         self._update_security_group_rules(load_balancer, sec_grp.get('id'))
 
     def failover_preparation(self, amphora):
+        if self.dns_integration_enabled:
+            self._failover_preparation(amphora)
+
+    def _failover_preparation(self, amphora):
         interfaces = self.get_plugged_networks(compute_id=amphora.compute_id)
 
         ports = []
@@ -478,17 +485,14 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
         for port in ports:
             try:
-
-                if self.dns_integration_enabled:
-                    self.neutron_client.update_port(port.id,
-                                                    {'port': {'dns_name': ''}})
+                self.neutron_client.update_port(port.id,
+                                                {'port': {'dns_name': ''}})
 
             except (neutron_client_exceptions.NotFound,
                     neutron_client_exceptions.PortNotFoundClient):
                 raise base.PortNotFound()
 
     def plug_port(self, amphora, port):
-        plugged_interface = None
         try:
             interface = self.nova_client.servers.interface_attach(
                 server=amphora.compute_id, net_id=None,
@@ -503,19 +507,19 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
             else:
                 raise base.PlugNetworkException(e.message)
         except nova_client_exceptions.Conflict:
-            LOG.info(_LI('Port %(portid)s is already plugged, '
-                     'skipping') % {'portid': port.id})
+            LOG.info('Port %(portid)s is already plugged, '
+                     'skipping', {'portid': port.id})
             plugged_interface = n_data_models.Interface(
                 compute_id=amphora.compute_id,
                 network_id=port.network_id,
                 port_id=port.id,
                 fixed_ips=port.fixed_ips)
         except Exception:
-            message = _LE('Error plugging amphora (compute_id: '
-                          '{compute_id}) into port '
-                          '{port_id}.').format(
-                              compute_id=amphora.compute_id,
-                              port_id=port.id)
+            message = _('Error plugging amphora (compute_id: '
+                        '{compute_id}) into port '
+                        '{port_id}.').format(
+                            compute_id=amphora.compute_id,
+                            port_id=port.id)
             LOG.exception(message)
             raise base.PlugNetworkException(message)
 
@@ -531,6 +535,7 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
                 vrrp_port = self.get_port(amp.vrrp_port_id)
                 vrrp_subnet = self.get_subnet(
                     vrrp_port.get_subnet_id(amp.vrrp_ip))
+                vrrp_port.network = self.get_network(vrrp_port.network_id)
                 ha_port = self.get_port(amp.ha_port_id)
                 ha_subnet = self.get_subnet(
                     ha_port.get_subnet_id(amp.ha_ip))
@@ -595,4 +600,4 @@ class AllowedAddressPairsDriver(neutron_base.BaseNeutronDriver):
 
             except (neutron_client_exceptions.NotFound,
                     neutron_client_exceptions.PortNotFoundClient):
-                    pass
+                pass
