@@ -24,6 +24,8 @@ class LoadBalancersClient(rest_client.RestClient):
     _LOAD_BALANCERS_URL = "v1/loadbalancers"
     _LOAD_BALANCER_URL = "{base_url}/{{lb_id}}".format(
         base_url=_LOAD_BALANCERS_URL)
+    _LOAD_BALANCER_CASCADE_DELETE_URL = "{lb_url}/delete_cascade".format(
+        lb_url=_LOAD_BALANCER_URL)
 
     def list_load_balancers(self, params=None):
         """List all load balancers."""
@@ -79,6 +81,13 @@ class LoadBalancersClient(rest_client.RestClient):
         self.expected_success(202, resp.status)
         return rest_client.ResponseBody(resp, body)
 
+    def delete_load_balancer_cascade(self, lb_id):
+        """Delete an existing load balancer (cascading)."""
+        url = self._LOAD_BALANCER_CASCADE_DELETE_URL.format(lb_id=lb_id)
+        resp, body = self.delete(url)
+        self.expected_success(202, resp.status)
+        return rest_client.ResponseBody(resp, body)
+
     def create_load_balancer_over_quota(self, **kwargs):
         """Attempt to build a load balancer over quota."""
         url = self._LOAD_BALANCERS_URL
