@@ -20,7 +20,7 @@ import six
 
 X509_CERT_CN = 'www.example.com'
 
-X509_CERT = """-----BEGIN CERTIFICATE-----
+X509_CERT = b"""-----BEGIN CERTIFICATE-----
 MIIE8TCCAtmgAwIBAgICEAEwDQYJKoZIhvcNAQELBQAwIzEhMB8GA1UEAwwYY2Et
 aW50QHNiYWx1a29mZi5pYm0uY29tMB4XDTE2MDkyNzA4MjkzNFoXDTI2MDkyNTA4
 MjkzNFowGjEYMBYGA1UEAwwPd3d3LmV4YW1wbGUuY29tMIIBIjANBgkqhkiG9w0B
@@ -50,7 +50,7 @@ Vi/XwUwVUqRURyQtuP8QJdPh9KD7uX6xHjqBALdwzCYAFaqelPue7TJ7R/I5+02A
 DV8BnY7U3zPtHtPf6i8vdYwgAOJG
 -----END CERTIFICATE-----"""
 
-X509_CERT_KEY = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY = b"""-----BEGIN RSA PRIVATE KEY-----
 MIIEogIBAAKCAQEA34asqEe1MexBKGmBcrco08LYYFfJjpmW8m1yKJsmS2nmHNhJ
 y4Fl+3cPDyHYOiVxnsaMIv1Q8ZMRpjYH2LhvzLt2doyMiiJrqA3ScdhZVlGKaURv
 ASSj9dmbRBMqdXZBvTZnMH4aSkL4DalU7NiW+jbMb5Gmf+bozE4ZAOES6eXsP5+y
@@ -78,7 +78,7 @@ nUBaaqVibLaROn4V1QnlSOA2vjc2jMMDKMfnjawtqBC018tQDVcE75sun7UzyxtS
 OWaQy6KhqrKpPy3tS1wt1vAYPWZw/EIo4dDXYBo55REI5mSBZrM=
 -----END RSA PRIVATE KEY-----"""
 
-X509_CERT_KEY_ENCRYPTED = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY_ENCRYPTED = b"""-----BEGIN RSA PRIVATE KEY-----
 Proc-Type: 4,ENCRYPTED
 DEK-Info: AES-256-CBC,086BA545587FF5F6F4DD9AACC122603A
 
@@ -113,7 +113,7 @@ X509_CERT_KEY_PASSPHRASE = """asdf"""
 
 X509_CERT_CN_2 = 'www2.example.com'
 
-X509_CERT_2 = """-----BEGIN CERTIFICATE-----
+X509_CERT_2 = b"""-----BEGIN CERTIFICATE-----
 MIIEbjCCAlagAwIBAgICEAIwDQYJKoZIhvcNAQELBQAwIzEhMB8GA1UEAwwYY2Et
 aW50QHNiYWx1a29mZi5pYm0uY29tMB4XDTE2MDkyOTIzNDk0MFoXDTI2MDkyNzIz
 NDk0MFowGzEZMBcGA1UEAwwQd3d3Mi5leGFtcGxlLmNvbTCBnzANBgkqhkiG9w0B
@@ -140,7 +140,7 @@ HULxFHp3QLrnbQEvPIcD0EWppJ1GMqb/Gv8jORzOks56UtOIfavrzGrcvRSKoC4Q
 lDApYKCiRvvBSVfgpoiVungh2NWSmNW5bn2uOkPt+vTjcA==
 -----END CERTIFICATE-----"""
 
-X509_CERT_KEY_2 = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY_2 = b"""-----BEGIN RSA PRIVATE KEY-----
 MIICXAIBAAKBgQCnyr3JsiVM/4CRrWNosTbaATvzi3etDQoLzc4NJjTbzdFBfoF8
 esBVOPVy3K/PQPqHFeJhfCL2Zdce8HPPFJw/EiZDuNu9jI/QK1qBDhAaPvD5dVZj
 3htjtflwXk2UYeXQQgcE7YbTBzjnkyDOVqSR+hruT1gkzfuF/CUFXohf1QIDAQAB
@@ -156,7 +156,7 @@ yNDu6ayAqhUGOTDVMqkCQG9Vk7xjpe8iLkI4h7PaxaqiSwY+pyY3QoErlumALffM
 t3c9Zw9YGbij+605loxv5jREFeSQMYgp2GK7rO7DTbI=
 -----END RSA PRIVATE KEY-----"""
 
-X509_CERT_KEY_ENCRYPTED_2 = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY_ENCRYPTED_2 = b"""-----BEGIN RSA PRIVATE KEY-----
 Proc-Type: 4,ENCRYPTED
 DEK-Info: AES-256-CBC,3CAEB474D1526248CA20B5E4F84A6BB7
 
@@ -180,7 +180,7 @@ X509_CERT_KEY_PASSPHRASE_2 = """asdf"""
 # Wildcard cert for testing
 X509_CERT_CN_3 = '*.www3.example.com'
 
-X509_CERT_3 = """-----BEGIN CERTIFICATE-----
+X509_CERT_3 = b"""-----BEGIN CERTIFICATE-----
 MIIFJTCCAw2gAwIBAgICEAUwDQYJKoZIhvcNAQELBQAwIzEhMB8GA1UEAwwYY2Et
 aW50QHNiYWx1a29mZi5pYm0uY29tMB4XDTE2MDkzMDE3MDkyNloXDTI2MDkyODE3
 MDkyNlowHTEbMBkGA1UEAwwSKi53d3czLmV4YW1wbGUuY29tMIIBIjANBgkqhkiG
@@ -211,7 +211,7 @@ LwW88v99ZsWWIkE6O22+MmJGs4kxPXBFhlDUCC9zPBn2UBK8dXSYL0+F3O7cjWQ7
 UUddoYPP4r24JRrqzBEldSDzWeNSORpUkg==
 -----END CERTIFICATE-----"""
 
-X509_CERT_KEY_3 = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY_3 = b"""-----BEGIN RSA PRIVATE KEY-----
 MIIEowIBAAKCAQEA6v/973etopk2Vz95DUcx8X6hLfJ5m8s+scn7nMZ37fSqAGPF
 0veGpqyqxorwh+GYLjlrvZkhVi7IZJAsLU2ztG4+MEoYzbyhgJerFmepBC7xPIJE
 jh8FKhtpvxVOMFcXJ1CZT89Ww0rVPnaoE09DS0DRo5s+lW0dD6TaQW0S/6RCZ5Rp
@@ -239,7 +239,7 @@ Mg0ePQKBgFn2yh/gKf0InDtR6IlIG9HVI+lMKxyU5iRH/9MQ7GS+sSjiAXdOtGJJ
 KX8jFiCL+HcZX+pqAaUuifgwnqd88EX7MPoU6Yjq02To9ZAPA+SA
 -----END RSA PRIVATE KEY-----"""
 
-X509_CERT_KEY_ENCRYPTED_3 = """-----BEGIN RSA PRIVATE KEY-----
+X509_CERT_KEY_ENCRYPTED_3 = b"""-----BEGIN RSA PRIVATE KEY-----
 Proc-Type: 4,ENCRYPTED
 DEK-Info: AES-256-CBC,088E3835B2238C332FC7AED391C9CF8D
 
@@ -278,7 +278,7 @@ X509_CERT_KEY_PASSPHRASE_3 = """asdf"""
 # intermediate certificate chains into barbican.
 
 X509_IMDS_LIST = [
-    """-----BEGIN CERTIFICATE-----
+    b"""-----BEGIN CERTIFICATE-----
 MIIFcjCCA1qgAwIBAgICEAAwDQYJKoZIhvcNAQELBQAwbTELMAkGA1UEBhMCVVMx
 EzARBgNVBAgMCldhc2hpbmd0b24xEDAOBgNVBAcMB1NlYXR0bGUxDDAKBgNVBAoM
 A0lCTTEpMCcGA1UEAwwgbWFzdGVyLWNhLXRlc3RAc2JhbHVrb2ZmLmlibS5jb20w
@@ -310,7 +310,7 @@ XO6erkwabZxCVfGgvIk9hE4x6+Cu+jdOLTpAwq1mcQroAp1+CInHrZeHdnhz0zR8
 sJlV015O7iIu22bowsDcF9RfvkdHNULrClWI12sRspXF9VmRjbDyG4eASBiulJQV
 bk9D26vP
 -----END CERTIFICATE-----""",
-    """-----BEGIN CERTIFICATE-----
+    b"""-----BEGIN CERTIFICATE-----
 MIIFwDCCA6igAwIBAgIJAJLWg/Z3x5xpMA0GCSqGSIb3DQEBCwUAMG0xCzAJBgNV
 BAYTAlVTMRMwEQYDVQQIDApXYXNoaW5ndG9uMRAwDgYDVQQHDAdTZWF0dGxlMQww
 CgYDVQQKDANJQk0xKTAnBgNVBAMMIG1hc3Rlci1jYS10ZXN0QHNiYWx1a29mZi5p
@@ -344,9 +344,9 @@ C6WXGJPCEOfOYsxdZMDbD7q9CqgT5P4kI8VfryB5iqaLfDtUwjT8GPoTybFiWHMk
 n3yPHeLbGBLg9jphH7MMmsn57Z9fYjJADOOLFKG+W6txAQV3
 -----END CERTIFICATE-----"""]
 
-X509_IMDS = '\n'.join(X509_IMDS_LIST)
+X509_IMDS = b'\n'.join(X509_IMDS_LIST)
 
-PKCS7_PEM = """This line of spam should be ignored, as should the next line.
+PKCS7_PEM = b"""This line of spam should be ignored, as should the next line.
 
 -----BEGIN PKCS7-----
 MIILZwYJKoZIhvcNAQcCoIILWDCCC1QCAQExADALBgkqhkiG9w0BBwGgggs6MIIF
@@ -487,7 +487,7 @@ PKCS7_DER = b64decode(
 # Keys for the above CA certs, logged here to make it simple to sign other
 # certs for testing purposes in the future.
 
-INTERMEDIATE_KEY = """-----BEGIN RSA PRIVATE KEY-----
+INTERMEDIATE_KEY = b"""-----BEGIN RSA PRIVATE KEY-----
 MIIJJwIBAAKCAgEAvComYtbfb1/CAbYjoDlx8Mk6OJY2Dqs/dgasiI+K/2CTR8Xi
 eyRzYjNvXruivy3SZXjpA11fptCw3IZ+qTH9f1Sef2aa+3lk6sikmj+c01WyDiNS
 LetDEKbEohm3H1Tye068MzSrcgV0RB8AvphLyLhMA0R3fA5YyRuSdgqUt0XvwoMU
@@ -539,7 +539,7 @@ qH3e8e1WlIfA7FAqE1Dtae97oV/5wM9qp1rnijwq5jlZX+AqYq7GQ8J5x2ypGhZX
 +N7I5RuaLjkJJs3i/EzCDwp8F3ZXZRiILaWSaGZlrZ8jgVtlNhNfVYVFuQ==
 -----END RSA PRIVATE KEY-----"""
 
-CA_KEY = """-----BEGIN RSA PRIVATE KEY-----
+CA_KEY = b"""-----BEGIN RSA PRIVATE KEY-----
 -----BEGIN RSA PRIVATE KEY-----
 MIIJKwIBAAKCAgEA3WLWYLlM0aaISfBiNvygl2cXehc5mbaqys9SYWcUleeloWJ2
 JY80IEbD1/JcVFgv50ptd3I4rXyhv07wzU7Om2/be1ZGDNJNHsLwK3DUUkkf2Bzt
@@ -595,7 +595,7 @@ r187A8Q9L5pB57JnuY9nO7MvrINJVNbLPYjanqrkqvwDjiPkzETVm50mVtFYLWgw
 # An expired self-signed cert for testing.
 X509_EXPIRED_CN = 'www.example.com'
 
-X509_EXPIRED = """-----BEGIN CERTIFICATE-----
+X509_EXPIRED = b"""-----BEGIN CERTIFICATE-----
 MIIDfzCCAmegAwIBAgIJAKByYqy5fAfLMA0GCSqGSIb3DQEBCwUAMFYxCzAJBgNV
 BAYTAlVTMQ0wCwYDVQQIDAREZWFkMRAwDgYDVQQHDAdUb3RhbGx5MQwwCgYDVQQK
 DANJQk0xGDAWBgNVBAMMD3d3dy5leGFtcGxlLmNvbTAeFw0xNjA5MjQxODAxNTRa
@@ -617,7 +617,7 @@ S+aDmoFsO3i/E+x+qm5H0swjU9dLCvdMjo0VUpk5f1aJJ10xpeKTUYOB55haalJI
 j+/EXRZyEna+vPrS8mCl0GMvlFm0ZWFdWaWPR7l3J/J4is0=
 -----END CERTIFICATE-----"""
 
-X509_EXPIRED_KEY = """-----BEGIN RSA PRIVATE KEY-----
+X509_EXPIRED_KEY = b"""-----BEGIN RSA PRIVATE KEY-----
 MIIEowIBAAKCAQEApleCh4ju3MwotSzQWn2oScgtlMfZePlD+PQpOQsPkDdUtE4m
 h/WUxvy+gCQXXqOcIL4DhG6mEQ5C0RQBTCVu9i/SutIRYpi2QYqkQCDuYfTn8GKI
 tK694hCJ2jq7jrn0UnduBg0T8/TFvy8y7HLgzskfU4XxdF4jTlR8uoavmKyOuL8h
@@ -646,7 +646,7 @@ Je8uvLnAPRLL95ZhclaSw2vAxmaiGIsm7WGhjnRQ2Vntgd6fNgY9
 -----END RSA PRIVATE KEY-----"""
 
 # Other certificates and keys used in tests.
-ALT_EXT_CRT = """-----BEGIN CERTIFICATE-----
+ALT_EXT_CRT = b"""-----BEGIN CERTIFICATE-----
 MIIGqjCCBZKgAwIBAgIJAIApBg8slSSiMA0GCSqGSIb3DQEBBQUAMIGLMQswCQYD
 VQQGEwJVUzEOMAwGA1UECAwFVGV4YXMxFDASBgNVBAcMC1NhbiBBbnRvbmlvMR4w
 HAYDVQQKDBVPcGVuU3RhY2sgRXhwZXJpbWVudHMxFjAUBgNVBAsMDU5ldXRyb24g
@@ -685,7 +685,7 @@ tL4Drm+OCXJwTrE7ClTMCwcrZnLl4tI+Z+X3DV92WQB8ldST/QFjz1hgs/4zrADA
 elu2c/X7MR4ObOjhDfaVGQ8kMhYf5hx69qyNDsGi
 -----END CERTIFICATE-----"""
 
-ALT_EXT_CRT_KEY = """
+ALT_EXT_CRT_KEY = b"""
 -----BEGIN RSA PRIVATE KEY-----
 MIIEowIBAAKCAQEAsvWeZsM9QOmzziLWwzeuEetz4OW7Q3/ApBYpkV6JZS0X+mi3
 X1XejTJcOmyDtblGQsxMWRkRydCnIZ2kAaNOPOY1cxnD30TPGyatHeXqFQQhKJ9V
@@ -716,7 +716,7 @@ iMwJYgm98P27s4TEMdhlPNVJrj1FrD+4VrgpOsoM20EkZnTvel9s
 
 ENCRYPTED_PKCS8_CRT_KEY_PASSPHRASE = 'test_passphrase'
 
-ENCRYPTED_PKCS8_CRT_KEY = """-----BEGIN ENCRYPTED PRIVATE KEY-----
+ENCRYPTED_PKCS8_CRT_KEY = b"""-----BEGIN ENCRYPTED PRIVATE KEY-----
 MIIE6TAbBgkqhkiG9w0BBQMwDgQIT04zko6pmJICAggABIIEyL/79sqzTQ7BsEjY
 ao2Uhh3//mpNJfCDhjSZOmWL7s4+161cEqpxrfxo4bHH8fkZ60VZUQP8CjwwQUhP
 4iwpv2bYbQwzlttZwTC6s28wh7FRtgVoVPTwvXJa6fl2zAjLtsjwLZ/556ez9xIJ
@@ -746,7 +746,7 @@ WwMJugHFk5NQuse3P4Hh9smQrRrv1dvnpt7s4yKStKolXUaFWcXJvXVaDfR5266Y
 p7cuYY1cAyI7gFfl5A==
 -----END ENCRYPTED PRIVATE KEY-----"""
 
-UNENCRYPTED_PKCS8_CRT_KEY = """-----BEGIN PRIVATE KEY-----
+UNENCRYPTED_PKCS8_CRT_KEY = b"""-----BEGIN PRIVATE KEY-----
 MIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQCy9Z5mwz1A6bPO
 ItbDN64R63Pg5btDf8CkFimRXollLRf6aLdfVd6NMlw6bIO1uUZCzExZGRHJ0Kch
 naQBo0485jVzGcPfRM8bJq0d5eoVBCEon1W7xLn7WGU+oz8TOTC+lgIxTWgRGT1r
@@ -777,7 +777,7 @@ P7hWuCk6ygzbQSRmdO96X2w=
 
 EXPECTED_IMD_TEST_SUBJS = ["IMD3", "IMD2", "IMD1"]
 
-TEST_X509_IMDS = """Junk
+TEST_X509_IMDS = b"""Junk
 -----BEGIN CERTIFICATE-----
 MIIBhDCCAS6gAwIBAgIGAUo7hO/eMA0GCSqGSIb3DQEBCwUAMA8xDTALBgNVBAMT
 BElNRDIwHhcNMTQxMjExMjI0MjU1WhcNMjUxMTIzMjI0MjU1WjAPMQ0wCwYDVQQD
